<?php

namespace jrosset\EnvReader;

use Exception;
use jrosset\Collections\InsensitiveCaseKeyCollection;

/**
 * A INI file configuration class
 */
abstract class IniConfig extends GenericConfig implements IExternalConfigFile {
    use TMultiLevelProperties;

    /**
     * @inheritDoc
     */
    protected function readConfig (): void {
        $configFile = $this->getConfigFilePath();
        if (!file_exists($configFile)) {
            throw new Exception('Unable to find configuration file "' . $configFile . '"');
        }

        if (($iniData = parse_ini_file($configFile, true, INI_SCANNER_RAW)) === false) {
            throw new Exception('Unable to read configuration file "' . $configFile . '"');
        }
        self::parseIniLevel($iniData, $this->properties);
    }
    /**
     * Parse an INI level to properties
     *
     * @param array                        $iniLevelData The INI level
     * @param InsensitiveCaseKeyCollection $properties   [OUT] The out properties list
     *
     * @return void
     */
    private static function parseIniLevel (array $iniLevelData, InsensitiveCaseKeyCollection $properties): void {
        foreach ($iniLevelData as $iniKey => $iniValue) {
            if (is_array($iniValue)) {
                self::parseIniLevel($iniValue, $subProperties = new InsensitiveCaseKeyCollection());
                $iniValue = $subProperties;
            }

            $properties->set($iniKey, $iniValue);
        }
    }
}