<?php

namespace jrosset\EnvReader\XmlStructure;

use jrosset\Collections\InsensitiveCaseKeyCollection;
use SimpleXMLElement;

/**
 * Structure about XML attributes
 */
class AttributesXmlStructure implements IXmlStructure {
    /**
     * @var array Allowed attributes. Empty for all
     */
    private array $allowed;
    /**
     * @var InsensitiveCaseKeyCollection Mapping between attributes name and properties name
     */
    private InsensitiveCaseKeyCollection $mapping;

    /**
     * Initialize
     */
    public function __construct () {
        $this->allowed = [];
        $this->mapping = new InsensitiveCaseKeyCollection();
    }

    /**
     * Add a new allowed attribute name
     *
     * @param string      $attributeName The new allowed attribute name
     * @param string|null $propertyName  The corresponding property name. Null if identical to attribute name
     *
     * @return $this
     */
    public function addAllowed (string $attributeName, ?string $propertyName = null): self {
        $this->allowed[] = mb_strtolower($attributeName);
        if ($propertyName !== null) {
            $this->addMapping($attributeName, $propertyName);
        }
        return $this;
    }
    /**
     * Add a new mapping
     *
     * @param string $attributeName The new attribute name
     * @param string $propertyName  The corresponding property name
     *
     * @return $this
     */
    public function addMapping (string $attributeName, string $propertyName): self {
        $this->mapping->set($attributeName, $propertyName);
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function parseXml (SimpleXMLElement $xmlNode): InsensitiveCaseKeyCollection {
        $properties = new InsensitiveCaseKeyCollection();
        if (($attributes = $xmlNode->attributes()) === null) {
            return $properties;
        }

        $allowAll = count($this->allowed) === 0;
        foreach ($attributes as $attributeName => $attributeValue) {
            if (!$allowAll && !in_array(mb_strtolower($attributeName), $this->allowed)) {
                continue;
            }
            $properties->set($this->mapping->get($attributeName) ?? $attributeName, (string)$attributeValue);
        }
        return $properties;
    }
}