<?php

namespace jrosset\EnvReader\XmlStructure;

use jrosset\Collections\InsensitiveCaseKeyCollection;
use SimpleXMLElement;

/**
 * Structure for XML node with text __and__ attributes
 */
class TextAndAttributesXmlStructure implements IXmlStructure {
    use TAttributeListXmlStructure;

    /**
     * @var string The property name for node's text
     */
    private string $textPropertyName;
    /**
     * @var TextXmlStructure The internal node's text
     */
    private TextXmlStructure $textXmlStructure;

    /**
     * Initialize
     *
     * @param string                $textPropertyName The property name for node's text
     * @param TextXmlStructure|null $textXmlStructure The node's text structure. Null if default
     */
    public function __construct (string $textPropertyName, ?TextXmlStructure $textXmlStructure = null) {
        $this->textPropertyName = $textPropertyName;
        $this->textXmlStructure = $textXmlStructure ?? new TextXmlStructure();
    }

    /**
     * @inheritDoc
     */
    public function parseXml (SimpleXMLElement $xmlNode): InsensitiveCaseKeyCollection {
        $properties = $this->parseXmlAttributes($xmlNode);
        $properties->set($this->textPropertyName, $this->textXmlStructure->parseXml($xmlNode));
        return $properties;
    }
}